@@ -9,7 +9,7 @@ package Rex::CLI;
 use strict;
 use warnings;
 
-our $VERSION = '1.6.0'; # VERSION
+our $VERSION = '1.12.2'; # VERSION
 
 use FindBin;
 use File::Basename qw(basename dirname);
@@ -284,7 +284,7 @@ CHECK_OVERWRITE: {
 
     Rex::global_sudo(0);
     Rex::Logger::debug("Removing lockfile") if ( !exists $opts{'F'} );
-    CORE::unlink("$::rexfile.lock") if ( !exists $opts{'F'} );
+    CORE::unlink("$::rexfile.lock")         if ( !exists $opts{'F'} );
     CORE::exit 0;
   }
 
@@ -428,6 +428,7 @@ sub __help__ {
   printf $fmt, "-Q",   "Really quiet: output nothing";
   print "\n";
   printf $fmt, "-T",  "List tasks";
+  printf $fmt, "-Ta", "List all tasks, including hidden";
   printf $fmt, "-Tm", "List tasks in machine-readable format";
   printf $fmt, "-Tv", "List tasks verbosely";
   printf $fmt, "-Ty", "List tasks in YAML format";
@@ -483,7 +484,14 @@ sub _handle_T {
 sub _list_tasks {
   Rex::Logger::debug("Listing Tasks");
 
-  my @tasks = Rex::TaskList->create()->get_tasks;
+  my @tasks;
+  if ( $opts{'a'} ) {
+    @tasks = sort Rex::TaskList->create()->get_all_tasks(qr/.*/);
+  }
+  else {
+    @tasks = Rex::TaskList->create()->get_tasks;
+  }
+
   if ( defined $ARGV[0] ) {
     @tasks = grep { $_ =~ /^$ARGV[0]/ } @tasks;
 
@@ -536,7 +544,7 @@ sub _list_batches {
 
   _print_color( "Batches\n", 'yellow' );
   my $max_batch_len = max map { length } @batchs;
-  my $fmt = " %-" . $max_batch_len . "s  %s\n";
+  my $fmt           = " %-" . $max_batch_len . "s  %s\n";
 
   for my $batch ( sort @batchs ) {
     my $description = Rex::Batch->get_desc($batch);
@@ -562,7 +570,7 @@ sub _list_envs {
 
   _print_color( "Environments\n", "yellow" ) if scalar @envs;
   my $max_env_len = max map { length $_->{name} } @envs;
-  my $fmt = " %-" . $max_env_len . "s  %s\n";
+  my $fmt         = " %-" . $max_env_len . "s  %s\n";
 
   for my $e ( sort @envs ) {
     my $output = sprintf $fmt, $e->{name}, $e->{description};
@@ -581,10 +589,10 @@ sub _list_groups {
 
   _print_color( "Server Groups\n", "yellow" );
   my $max_group_len = max map { length } @group_names;
-  my $fmt = " %-" . $max_group_len . "s  %s\n";
+  my $fmt           = " %-" . $max_group_len . "s  %s\n";
 
   for my $group_name (@group_names) {
-    my $hosts = join( ", ", sort @{ $groups{$group_name} } );
+    my $hosts  = join( ", ", sort @{ $groups{$group_name} } );
     my $output = sprintf $fmt, $group_name, $hosts;
     my $indent = " " x $max_group_len . "   ";
     print wrap( "", $indent, $output );
@@ -758,7 +766,7 @@ sub load_rexfile {
     if ($stderr) {
       my @lines = split( $/, $stderr );
       Rex::Logger::info( "You have some code warnings:", 'warn' );
-      Rex::Logger::info( "\t$_", 'warn' ) for @lines;
+      Rex::Logger::info( "\t$_",                         'warn' ) for @lines;
     }
 
     1;
@@ -775,7 +783,7 @@ sub load_rexfile {
     my @lines = split( $/, $e );
 
     Rex::Logger::info( "Compile time errors:", 'error' );
-    Rex::Logger::info( "\t$_", 'error' ) for @lines;
+    Rex::Logger::info( "\t$_",                 'error' ) for @lines;
 
     exit 1;
   }
@@ -788,7 +796,7 @@ sub exit_rex {
 
   Rex::global_sudo(0);
   Rex::Logger::debug("Removing lockfile") if !exists $opts{'F'};
-  unlink("$::rexfile.lock") if !exists $opts{'F'};
+  unlink("$::rexfile.lock")               if !exists $opts{'F'};
 
   select STDOUT;
 
@@ -807,7 +815,7 @@ sub exit_rex {
     my @exit_codes = Rex::TaskList->create()->get_exit_codes();
     for my $exit_code (@exit_codes) {
       $exit_code = $exit_code >> 8 if $exit_code > 255;
-      CORE::exit($exit_code) if $exit_code != 0;
+      CORE::exit($exit_code)       if $exit_code != 0;
     }
   }
 
