@@ -9,7 +9,7 @@ package Rex::Interface::File::Local;
 use strict;
 use warnings;
 
-our $VERSION = '1.8.1'; # VERSION
+our $VERSION = '1.12.2'; # VERSION
 
 use Rex::Interface::File::Base;
 use base qw(Rex::Interface::File::Base);
@@ -45,6 +45,9 @@ sub read {
 sub write {
   my ( $self, $buf ) = @_;
 
+  utf8::encode($buf)
+    if Rex::Config->get_write_utf8_files && utf8::is_utf8($buf);
+
   my $fh = $self->{fh};
   print $fh $buf;
 }
